@@ -23,27 +23,44 @@
 #if !defined(__gda_command_h__)
 #  define __gda_command_h__
 
-#include <libgda/GNOME_Database.h>
 #include <glib/gmacros.h>
+#include <glib/gtypes.h>
 
 G_BEGIN_DECLS
 
-typedef GNOME_Database_Command GdaCommand;
 typedef enum {
-	GDA_COMMAND_TYPE_SQL = GNOME_Database_COMMAND_TYPE_SQL,
-	GDA_COMMAND_TYPE_XML = GNOME_Database_COMMAND_TYPE_XML,
-	GDA_COMMAND_TYPE_PROCEDURE = GNOME_Database_COMMAND_TYPE_PROCEDURE,
-	GDA_COMMAND_TYPE_TABLE = GNOME_Database_COMMAND_TYPE_TABLE,
-	GDA_COMMAND_TYPE_INVALID = GNOME_Database_COMMAND_TYPE_INVALID
+	GDA_COMMAND_OPTION_IGNORE_ERRORS  = 1,
+	GDA_COMMAND_OPTION_STOP_ON_ERRORS = 1 << 1,
+	GDA_COMMAND_OPTION_BAD_OPTION     = 1 << 2
+} GdaCommandOptions;
+
+#define GDA_COMMAND_DEFAULT_OPTION GDA_COMMAND_OPTION_IGNORE_ERRORS
+
+typedef enum {
+	GDA_COMMAND_TYPE_SQL,
+	GDA_COMMAND_TYPE_XML,
+	GDA_COMMAND_TYPE_PROCEDURE,
+	GDA_COMMAND_TYPE_TABLE,
+	GDA_COMMAND_TYPE_SCHEMA,
+	GDA_COMMAND_TYPE_INVALID
 } GdaCommandType;
 
-GdaCommand    *gda_command_new (const gchar *text, GdaCommandType type);
+typedef struct {
+	gchar *text;
+	GdaCommandType type;
+	GdaCommandOptions options;
+} GdaCommand;
+
+GdaCommand    *gda_command_new (const gchar *text, GdaCommandType type,
+				GdaCommandOptions options);
 void           gda_command_free (GdaCommand *cmd);
 
 const gchar   *gda_command_get_text (GdaCommand *cmd);
 void           gda_command_set_text (GdaCommand *cmd, const gchar *text);
 GdaCommandType gda_command_get_command_type (GdaCommand *cmd);
 void           gda_command_set_command_type (GdaCommand *cmd, GdaCommandType type);
+GdaCommandOptions gda_command_get_options (GdaCommand *cmd);
+void           gda_command_set_options (GdaCommand *cmd, GdaCommandOptions options);
 
 G_END_DECLS
 
