@@ -42,6 +42,8 @@ typedef struct {
     gchar *name;
     gchar *type;
     gchar *value;
+    gchar *mtime;
+    gchar *muser;
 } gda_config_entry;
 
 typedef struct {
@@ -85,6 +87,14 @@ gda_config_read_entries (xmlNodePtr cur)
 			if (entry->value == NULL)
 				g_warning ("NULL 'value' in an entry");
 
+			entry->muser =  xmlGetProp(cur, "muser");
+			if (entry->value == NULL)
+				g_warning ("NULL 'muser' in an entry");
+
+			entry->mtime =  xmlGetProp(cur, "mtime");
+			if (entry->value == NULL)
+				g_warning ("NULL 'mtime' in an entry");
+
 			list = g_list_append (list, entry);
 		} else {
 			g_warning ("'entry' expected, got '%s'. Ignoring...", 
@@ -270,7 +280,7 @@ gda_config_add_entry (const gchar *section_path,
 	entry->type = g_strdup (type);
 	entry->value = g_strdup (value);
 	
-	section = gda_config_search_section (cfg_client->global, section_path);
+	section = gda_config_search_section (cfg_client->user, section_path);
 	if (section == NULL)
 		section = gda_config_add_section (section_path);
 
@@ -299,6 +309,74 @@ free_section (gpointer data, gpointer user_data)
 	g_free (section);
 }
 
+static void
+add_xml_entry (xmlNodePtr parent, gda_config_entry *entry)
+{
+	xmlNodePtr new_node;
+
+	new_node = xmlNewTextChild (parent, NULL, "entry", NULL);
+	xmlSetProp (new_node, "name", entry->name ? entry->name : "");
+	xmlSetProp (new_node, "type", entry->type ? entry->type : "");
+	xmlSetProp (new_node, "value", entry->value ? entry->value : "");
+	xmlSetProp (new_node, "mtime", entry->name ? entry->name : "");
+	xmlSetProp (new_node, "muser", entry->type ? entry->type : "");
+}
+
+static xmlNodePtr
+add_xml_section (xmlNodePtr parent, gda_config_section *section)
+{
+	xmlNodePtr new_node;
+
+	new_node = xmlNewTextChild (parent, NULL, "section", NULL);
+	xmlSetProp (new_node, "path", section->path ? section->path : "");
+	return new_node;
+}
+
+static void
+write_config_file ()
+{
+	gda_config_client *cfg_client;
+	xmlDocPtr doc;
+	xmlNodePtr root;
+	xmlNodePtr xml_section;
+	GList *ls; /* List of sections */
+	GList *le; /* List of entries */
+	gda_config_section *section;
+	gda_config_entry *entry;
+	gchar *user_config;
+
+	cfg_client = get_config_client ();
+	g_return_if_fail (cfg_client->user != NULL);
+
+	doc = xmlNewDoc ("1.0");
+	g_return_if_fail (doc != NULL);
+	root = xmlNewDocNode (doc, NULL, "libgda-config", NULL);
+	xmlDocSetRootElement (doc, root);
+	for (ls = cfg_client->user; ls; ls = ls->next){
+		section = ls->data;
+		if (section == NULL)
+			continue;
+
+		xml_section = add_xml_section (root, section);
+		for (le = section->entries; le; le = le->next){
+			entry = le->data;
+			if (entry == NULL)
+				continue;
+
+			add_xml_entry (xml_section, le->data);
+		}
+	}
+
+	user_config = g_strdup_printf ("%s%s", g_get_home_dir (),
+					LIBGDA_USER_CONFIG_FILE);
+	if (xmlSaveFormatFile (user_config, doc, TRUE) == -1){
+		g_warning ("Error saving config data to %s", user_config);
+	}
+
+	g_free (user_config);
+	xmlFreeDoc (doc);
+}
+
 /*
  * Public functions
  */
@@ -446,6 +524,7 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
 		entry->value = g_strdup (new_value);
 	}
 
+	write_config_file ();
 	do_notify (path);
 }
 
@@ -489,6 +568,7 @@ gda_config_set_int (const gchar *path, gint new_value)
 		entry->value = g_strdup_printf ("%d", new_value);
 	}
 
+	write_config_file ();
 	do_notify (path);
 }
 
@@ -532,6 +612,7 @@ gda_config_set_float (const gchar * path, gdouble new_value)
 		entry->value = g_strdup_printf ("%f", new_value);
 	}
 
+	write_config_file ();
 	do_notify (path);
 }
 
@@ -575,6 +656,7 @@ gda_config_set_boolean (const gchar *path, gboolean new_value)
 		entry->value = g_strdup_printf ("%d", new_value);
 	}
 
+	write_config_file ();
 	do_notify (path);
 }
 
@@ -601,6 +683,7 @@ gda_config_remove_section (const gchar *path)
 
 	cfg_client->user = g_list_remove (cfg_client->user, section);
 	free_section (section, NULL);
+	write_config_file ();
 	do_notify (path);
 }
 
@@ -647,6 +730,7 @@ gda_config_remove_key (const gchar *path)
 	if (entry != NULL){
 		section->entries = g_list_remove (section->entries, entry);
 		free_entry (entry, NULL);
+		write_config_file ();
 		do_notify (path);
 	}
 }
@@ -865,11 +949,11 @@ gda_config_get_provider_list (void)
 		}
 
 		g_module_symbol (handle, "plugin_get_name",
-				 (gpointer) &plugin_get_name);
+				 (gpointer *) &plugin_get_name);
 		g_module_symbol (handle, "plugin_get_description",
-				 (gpointer) &plugin_get_description);
+				 (gpointer *) &plugin_get_description);
 		g_module_symbol (handle, "plugin_get_connection_params",
-				 (gpointer) &plugin_get_cnc_params);
+				 (gpointer *) &plugin_get_cnc_params);
 
 		info = g_new0 (GdaProviderInfo, 1);
 		info->location = path;
@@ -1122,7 +1206,7 @@ gda_config_get_data_source_model (void)
 		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->description));
 		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->username));
 
-		gda_data_model_array_append_row (GDA_DATA_MODEL_ARRAY (model), value_list);
+		gda_data_model_append_row (GDA_DATA_MODEL (model), value_list);
 	}
 	
 	/* free memory */
