@@ -31,17 +31,20 @@
 #include <libgda/gda-connection.h>
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-data-model-array.h>
+#include <libgda/gda-data-model-hash.h>
 #include <libgda/gda-data-model-list.h>
 #include <libgda/gda-error.h>
 #include <libgda/gda-export.h>
+#include <libgda/gda-field.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-parameter.h>
 #include <libgda/gda-quark-list.h>
-#include <libgda/gda-recordset.h>
 #include <libgda/gda-row.h>
+#include <libgda/gda-select.h>
 #include <libgda/gda-table.h>
 #include <libgda/gda-util.h>
 #include <libgda/gda-value.h>
+#include <libgda/gda-xml-connection.h>
 #include <libgda/gda-xml-document.h>
 #include <libgda/gda-xml-database.h>
 
