@@ -24,6 +24,7 @@
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
+#include <libgda/gda-data-model-array.h>
 #include "gda-postgres.h"
 #include "gda-postgres-provider.h"
 
@@ -41,6 +42,13 @@ static gboolean gda_postgres_provider_open_connection (GdaServerProvider *provid
 static gboolean gda_postgres_provider_close_connection (GdaServerProvider *provider,
 							GdaConnection *cnc);
 
+static gboolean gda_postgres_provider_create_database (GdaServerProvider *provider,
+						       GdaConnection *cnc,
+						       const gchar *name);
+static gboolean gda_postgres_provider_drop_database (GdaServerProvider *provider,
+						     GdaConnection *cnc,
+						     const gchar *name);
+
 static GList *gda_postgres_provider_execute_command (GdaServerProvider *provider,
 						     GdaConnection *cnc,
 						     GdaCommand *cmd,
@@ -109,6 +117,8 @@ gda_postgres_provider_class_init (GdaPostgresProviderClass *klass)
 	object_class->finalize = gda_postgres_provider_finalize;
 	provider_class->open_connection = gda_postgres_provider_open_connection;
 	provider_class->close_connection = gda_postgres_provider_close_connection;
+	provider_class->create_database = gda_postgres_provider_create_database;
+	provider_class->drop_database = gda_postgres_provider_drop_database;
 	provider_class->execute_command = gda_postgres_provider_execute_command;
 	provider_class->begin_transaction = gda_postgres_provider_begin_transaction;
 	provider_class->commit_transaction = gda_postgres_provider_commit_transaction;
@@ -379,8 +389,9 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 
 		while (arr[n]) {
 			PGresult *pg_res;
-			GdaRecordset *recset;
+			GdaDataModel *recset;
 			gint status;
+			gint i;
 
 			pg_res = PQexec(pconn, arr[n]);
 			if (pg_res == NULL) {
@@ -397,9 +408,13 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 			    status == PGRES_TUPLES_OK 			||
 			    status == PGRES_COMMAND_OK) {
 				recset = gda_postgres_recordset_new (cnc, pg_res);
-				if (GDA_IS_RECORDSET (recset)) {
-					gda_recordset_set_command_text (recset, arr[n]);
-					gda_recordset_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+				if (GDA_IS_DATA_MODEL (recset)) {
+					gda_data_model_set_command_text (recset, arr[n]);
+					gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+					for (i = PQnfields (pg_res) - 1; i >= 0; i--)
+						gda_data_model_set_column_title (recset, i, 
+									PQfname (pg_res, i));
+
 					reclist = g_list_append (reclist, recset);
 				}
 			}
@@ -420,6 +435,48 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 	return reclist;
 }
 
+/* create_database handler for the GdaPostgresProvider class */
+static gboolean
+gda_postgres_provider_create_database (GdaServerProvider *provider,
+				       GdaConnection *cnc,
+				       const gchar *name)
+{
+	gboolean retval;
+	gchar *sql;
+
+	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	sql = g_strdup_printf ("CREATE DATABASE %s", name);
+	retval = gda_postgres_provider_single_command (pg_prv, cnc, sql);
+	g_free (sql);
+
+	return retval;
+}
+
+/* drop_database handler for the GdaPostgresProvider class */
+static gboolean
+gda_postgres_provider_drop_database (GdaServerProvider *provider,
+				     GdaConnection *cnc,
+				     const gchar *name)
+{
+	gboolean retval;
+	gchar *sql;
+
+	GdaPostgresProvider *pg_prv = (GdaPostgresProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pg_prv), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	sql = g_strdup_printf ("DROP DATABASE %s", name);
+	retval = gda_postgres_provider_single_command (pg_prv, cnc, sql);
+	g_free (sql);
+
+	return retval;
+}
+
 /* execute_command handler for the GdaPostgresProvider class */
 static GList *
 gda_postgres_provider_execute_command (GdaServerProvider *provider,
@@ -532,8 +589,16 @@ static gboolean gda_postgres_provider_supports (GdaServerProvider *provider,
 	g_return_val_if_fail (GDA_IS_POSTGRES_PROVIDER (pgprv), FALSE);
 
 	switch (feature) {
+	case GDA_CONNECTION_FEATURE_AGGREGATES :
+	case GDA_CONNECTION_FEATURE_INDEXES :
+	case GDA_CONNECTION_FEATURE_INHERITANCE :
+	case GDA_CONNECTION_FEATURE_PROCEDURES :
+	case GDA_CONNECTION_FEATURE_SEQUENCES :
 	case GDA_CONNECTION_FEATURE_SQL :
 	case GDA_CONNECTION_FEATURE_TRANSACTIONS :
+	case GDA_CONNECTION_FEATURE_TRIGGERS :
+	case GDA_CONNECTION_FEATURE_USERS :
+	case GDA_CONNECTION_FEATURE_VIEWS :
 		return TRUE;
 	default :
 	}
@@ -545,7 +610,7 @@ static GdaDataModel *
 get_postgres_procedures (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -556,17 +621,17 @@ get_postgres_procedures (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_tables (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -579,10 +644,12 @@ get_postgres_tables (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Tables"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static void
@@ -603,7 +670,7 @@ add_string_row (gpointer data, gpointer user_data)
 	list.next = NULL;
 	list.prev = NULL;
 
-	gda_data_model_array_append_row (recset, &list);
+	gda_data_model_append_row (GDA_DATA_MODEL (recset), &list);
 
 	gda_value_free (value);
 }
@@ -619,10 +686,7 @@ get_postgres_types (GdaConnection *cnc, GdaParameterList *params)
 
 	/* create the recordset */
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (1));
-	//gda_server_recordset_model_set_field_defined_size (recset, 0, 32);
-	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
-	//gda_server_recordset_model_set_field_scale (recset, 0, 0);
-	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_VALUE_TYPE_STRING);
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Types"));
 
 	/* fill the recordset */
 	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_POSTGRES_HANDLE);
@@ -637,7 +701,7 @@ static GdaDataModel *
 get_postgres_views (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -650,17 +714,19 @@ get_postgres_views (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Views"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_indexes (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -673,17 +739,19 @@ get_postgres_indexes (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Indexes"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_aggregates (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -697,17 +765,19 @@ get_postgres_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Aggregates"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_triggers (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -720,19 +790,19 @@ get_postgres_triggers (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Triggers"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModelArray *
 gda_postgres_init_md_recset (GdaConnection *cnc)
 {
 	GdaDataModelArray *recset;
-	GdaValueType data_type;
-	gint defined_size;
-	int i;
+	gint i;
 	GdaPostgresColData cols[8] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING  },
@@ -744,17 +814,9 @@ gda_postgres_init_md_recset (GdaConnection *cnc)
 		{ N_("References")	, GDA_VALUE_TYPE_STRING  }
 		};
 
-	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (8));
-	for (i = 0; i < sizeof cols / sizeof cols[0]; i++) {
-		data_type = cols[i].data_type;
-		defined_size =  (data_type == GDA_VALUE_TYPE_STRING) ? NAMEDATALEN : 
-				(data_type == GDA_VALUE_TYPE_INTEGER) ? sizeof(gint) : 1;
-
-		//gda_server_recordset_model_set_field_defined_size (recset, i, defined_size);
+	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (sizeof cols / sizeof cols[0]));
+	for (i = 0; i < sizeof cols / sizeof cols[0]; i++)
 		gda_data_model_set_column_title (GDA_DATA_MODEL (recset), i, _(cols[i].col_name));
-		//gda_server_recordset_model_set_field_scale (recset, i, 0);
-		//gda_server_recordset_model_set_field_gdatype (recset, i, data_type);
-	}
 
 	return recset;
 }
@@ -1032,7 +1094,7 @@ add_g_list_row (gpointer data, gpointer user_data)
 	GList *rowlist = data;
 	GdaDataModelArray *recset = user_data;
 
-	gda_data_model_array_append_row (recset, rowlist);
+	gda_data_model_append_row (GDA_DATA_MODEL (recset), rowlist);
 	g_list_foreach (rowlist, (GFunc) gda_value_free, NULL);
 	g_list_free (rowlist);
 }
@@ -1069,7 +1131,7 @@ static GdaDataModel *
 get_postgres_databases (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -1082,17 +1144,19 @@ get_postgres_databases (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Databases"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_users (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -1105,17 +1169,19 @@ get_postgres_users (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Users"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 static GdaDataModel *
 get_postgres_sequences (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
-	GdaRecordset *recset;
+	GdaDataModel *recset;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -1131,10 +1197,12 @@ get_postgres_sequences (GdaConnection *cnc, GdaParameterList *params)
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_RECORDSET (reclist->data);
+	recset = GDA_DATA_MODEL (reclist->data);
 	g_list_free (reclist);
+	// Set it here instead of the SQL query to allow i18n
+	gda_data_model_set_column_title (recset, 0, _("Sequences"));
 
-	return GDA_DATA_MODEL (recset);
+	return recset;
 }
 
 /* get_schema handler for the GdaPostgresProvider class */
