@@ -20,10 +20,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+#include <libgda/gda-intl.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
 #include <libgda-report/gda-report-document.h>
-#include <bonobo/bonobo-i18n.h>
 
 #define PARENT_TYPE GDA_TYPE_XML_DOCUMENT
 
@@ -125,6 +126,24 @@ gda_report_document_new (void)
 GdaReportDocument *
 gda_report_document_new_from_string (const gchar *xml)
 {
+	GdaReportDocument *document;
+
+	g_return_val_if_fail (xml != NULL, NULL);
+
+	/* parse the XML string */
+	document = g_object_new (GDA_TYPE_REPORT_DOCUMENT, NULL);
+
+	GDA_XML_DOCUMENT (document)->doc = xmlParseMemory (xml, strlen (xml));
+	if (!GDA_XML_DOCUMENT (document)->doc) {
+		gda_log_error (_("Could not parse XML document"));
+		g_object_unref (G_OBJECT (document));
+		return NULL;
+	}
+
+	GDA_XML_DOCUMENT (document)->root =
+		xmlDocGetRootElement (GDA_XML_DOCUMENT (document)->doc);
+
+	return document;
 }
 
 /**
